import os
from hashlib import sha256
from datetime import datetime
def sanboxConfig():
    config = {
        'memory': '100M', # MB 
        'memory_swap': '100M', # MB
        'timeout': 3, # seconds
        'type': 'python3', 
        'dependency': 'todo',
        'cpu': 1,
        'pids_limit': 10,
        'host_path': os.getcwd(),
        'container_path': '/tmp',
        'image': 'library/python'
    }
    return config


from .box import Box
from .config import sanboxConfig
import os 

class Challenge(object):
    def __init__(self, filename: str, code: str, challenge_input=None, host_path=None, timeout=None, cpu=None, pids_limit=None, memory=None):
        self.filename = filename
        self.code = code
        self.input = challenge_input
        self.host_path = host_path
        self.timeout = timeout
        self.cpu = cpu
        self.pids_limit = pids_limit
        self.memory = memory

    def initBox(self, config=sanboxConfig()):
        config['filename'] = self.filename 
        config['code'] = self.code
        config['input'] = self.input
        if self.host_path:
            config['host_path'] = self.host_path
        if self.cpu:
            config['cpu'] = self.cpu
        if self.timeout:
            config['timeout'] = self.timeout
        if self.pids_limit:
            config['pids_limit'] = self.pids_limit
        if self.memory:
            config['memory'] = self.memory
        # initialize the sandbox
        self.box = Box(config)

    def getDir(self) -> str:
        return self.host_path

    def getName(self) -> str: 
        return self.filename

    def getCode(self) -> str:
        return self.code


import os
import sys
from datetime import datetime
import subprocess
import threading
from hashlib import sha256

class Box(object):
    def __init__(self, config):
        self.filename = sha256((datetime.now().strftime("%Y%m%d%H%M%S")+config['filename']).encode('utf-8')).hexdigest()[-25:]+'_'+config['filename']
        self.box_name = self.filename[:-3]
        # user's code
        self.code = config['code']
        # std_input
        self.input = config['input']
        self.input_name = 'input_' + self.filename[:-3]
        self.memory = config['memory']
        self.timeout = config['timeout']
        self.type = config['type']
        self.dependency = config['dependency']
        self.cpu = config['cpu']
        self.pids_limit = config['pids_limit']
        self.host_path = config['host_path']
        self.container_path = config['container_path']
        self.image = config['image']

    def check_path_exist(self):
        assert os.path.exists(self.host_path), f'Path {self.host_path} not found'
        return True

    def prepare(self):
        if self.check_path_exist():
            # code file
            with open(self.host_path+'/'+self.filename, 'w') as f:
                f.write(self.code)
            # challenge input
            if self.input:
                with open(self.host_path+'/'+self.input_name, 'w') as f:
                    f.write(self.input)
        return 'create file success'

    def run(self):
        #TODO: create code file
        print(self.prepare())

        #TODO: docker run parameters
        cmd = (
            f"/usr/bin/docker run --rm -a stdout -a stderr "
            f"--memory {self.memory} "
            f"--cpus {self.cpu} "
            f"--pids-limit {self.pids_limit} "
            f"-v {self.host_path}:{self.container_path} "
            f"--name {self.box_name} "
        )
        if self.input:
            cmd = cmd + f"{self.image} /bin/sh -c \"cat /tmp/{self.input_name} | {self.type} /tmp/{self.filename}\"" # cat "/tmp/inputname | python3 "/tmp/filename"
        else:
            cmd = cmd + f"{self.image} {self.type} /tmp/{self.filename}" # python3 "/tmp/filename"

        #TODO:return result
        try:
            def kill_docker():
                nonlocal timeout_flag
                try:  # catch race 
                    timeout_flag = True
                    subprocess.check_output(f"/usr/bin/docker kill {self.box_name}", stderr=subprocess.STDOUT, shell=True)
                except:
                    return "are you kidding me?"
            timeout_flag = False
            finished_flag = False
            t = threading.Timer(self.timeout, kill_docker)
            t.start()
            out_bytes = subprocess.check_output(cmd, stderr=subprocess.STDOUT, shell=True)
            try: # container killed 
                print('yes')   
                t.cancel()
            except:
                pass
            return out_bytes.decode().strip()
        except subprocess.CalledProcessError as e: # timeout or error
            if timeout_flag: # timeout
                return f'timeout! you only have {self.timeout} seconds'
            # other error
            try: # container killed 
                print('yes')   
                t.cancel()
            except:
                pass
            out_bytes = e.output       # Output generated before error
            code      = e.returncode   # Return code
            return out_bytes.decode().strip().replace('File "/tmp/{}", '.format(self.filename), '')

    def clear_file(self):
        os.system(f"rm {self.host_path}/{self.filename}")
        if self.input:
            os.system(f"rm {self.host_path}/{self.input_name}")




#!/usr/bin/env python
from setuptools import setup

setup(
    name='little-sandbox-py',
    version='1.0.6',
    description='A sandbox for running python scripts',
    url='https://github.com/Hustcw/Little-Sandbox',
    author='hustcw',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='sandbox docker python',
    packages=['sandbox'],
)
